use galactica_content::{Content, ContentIndex};
use galactica_playeragent::PlayerAgent;
use galactica_system::data::ShipPersonality;
use galactica_system::phys::{PhysImage, PhysSim, PhysSimShipHandle, PhysStepResources};
use galactica_util::timing::Timing;
use nalgebra::Point2;
use std::sync::Arc;
use std::time::Instant;

pub struct Game {
	// Core game data
	ct: Arc<Content>,
	phys_sim: PhysSim,
	timing: Timing,
	start_instant: Instant,

	// Metadata
	time_scale: f32,
	last_update: Instant,
}

unsafe impl<'a> Send for Game {}

impl<'a> Game {
	pub fn make_player(&mut self) -> PhysSimShipHandle {
		let player = self.phys_sim.add_ship(
			self.ct
				.ships
				.get(&ContentIndex::new("gypsum"))
				.unwrap()
				.clone(),
			self.ct
				.factions
				.get(&ContentIndex::new("player"))
				.unwrap()
				.clone(),
			ShipPersonality::Player,
			Point2::new(0.0, 4000.0),
		);

		let s = self.phys_sim.get_ship_mut(&player).unwrap();
		s.add_outfits([
			self.ct
				.outfits
				.get(&ContentIndex::new("plasma engines"))
				.unwrap()
				.clone(),
			self.ct
				.outfits
				.get(&ContentIndex::new("shield generator"))
				.unwrap()
				.clone(),
			self.ct
				.outfits
				.get(&ContentIndex::new("blaster"))
				.unwrap()
				.clone(),
			self.ct
				.outfits
				.get(&ContentIndex::new("blaster"))
				.unwrap()
				.clone(),
		]);

		return player;
	}

	pub fn new(ct: Arc<Content>) -> Self {
		let mut phys_sim = PhysSim::new();

		let a = phys_sim.add_ship(
			ct.ships.get(&ContentIndex::new("gypsum")).unwrap().clone(),
			ct.factions
				.get(&ContentIndex::new("enemy"))
				.unwrap()
				.clone(),
			ShipPersonality::Point,
			Point2::new(1000.0, 4000.0),
		);

		let s = phys_sim.get_ship_mut(&a).unwrap();
		s.add_outfits([
			ct.outfits
				.get(&ContentIndex::new("plasma engines"))
				.unwrap()
				.clone(),
			ct.outfits
				.get(&ContentIndex::new("shield generator"))
				.unwrap()
				.clone(),
			ct.outfits
				.get(&ContentIndex::new("blaster"))
				.unwrap()
				.clone(),
		]);

		let a = phys_sim.add_ship(
			ct.ships.get(&ContentIndex::new("gypsum")).unwrap().clone(),
			ct.factions
				.get(&ContentIndex::new("player"))
				.unwrap()
				.clone(),
			ShipPersonality::Dummy,
			Point2::new(200.0, 2000.0),
		);

		let s = phys_sim.get_ship_mut(&a).unwrap();
		s.add_outfits([
			ct.outfits
				.get(&ContentIndex::new("plasma engines"))
				.unwrap()
				.clone(),
			ct.outfits
				.get(&ContentIndex::new("shield generator"))
				.unwrap()
				.clone(),
			ct.outfits
				.get(&ContentIndex::new("blaster"))
				.unwrap()
				.clone(),
		]);

		Game {
			ct,
			phys_sim,
			timing: Timing::new(),
			start_instant: Instant::now(),
			last_update: Instant::now(),
			time_scale: 1.0,
		}
	}

	pub fn update_player_controls(&mut self, player: &mut PlayerAgent) {
		self.phys_sim.update_player_controls(player)
	}

	pub fn step(&mut self, phys_img: &PhysImage) {
		self.timing.start_frame();
		self.phys_sim.step(
			PhysStepResources {
				ct: self.ct.clone(),
				t: self.last_update.elapsed().as_secs_f32() * self.time_scale,
				timing: &mut self.timing,
			},
			phys_img,
		);

		self.last_update = Instant::now();
		self.timing.mark_frame();
	}

	pub fn update_image(&self, phys_img: &mut PhysImage) {
		self.phys_sim.update_image(phys_img);
	}
}

impl Game {
	pub fn get_timing(&self) -> &Timing {
		&self.timing
	}

	pub fn get_current_time(&self) -> f32 {
		self.start_instant.elapsed().as_secs_f32()
	}
}
