use std::sync::Arc;

use galactica_content::{Content, ContentIndex, SystemObject};
use rapier2d::geometry::ColliderHandle;

use crate::{camera::Camera, inputstatus::InputStatus, PlayerStatus};

/// What the player has selected
#[derive(Debug, Clone)]
pub enum PlayerSelection {
	/// We have nothing selected
	None,

	/// We have a system body selected
	OrbitingBody(Arc<SystemObject>),

	/// We have a ship selected
	Ship(ColliderHandle),
}

impl PlayerSelection {
	pub fn get_planet(&self) -> Option<&Arc<SystemObject>> {
		match self {
			Self::OrbitingBody(h) => Some(h),
			_ => None,
		}
	}
}

#[derive(Debug)]
pub struct PlayerAgent {
	/// Which ship this player is controlling
	pub ship: Option<ColliderHandle>,

	/// What the player has selected
	pub selection: PlayerSelection,

	/// This player's camera
	pub camera: Camera,

	/// What buttons this player is pressing
	pub input: InputStatus,
}

impl PlayerAgent {
	pub fn new(ct: &Content, ship: ColliderHandle) -> Self {
		Self {
			input: InputStatus::new(),
			camera: Camera::new(),
			ship: Some(ship),
			selection: PlayerSelection::OrbitingBody(
				ct.systems
					.values()
					.next()
					.unwrap()
					.objects
					.get(&ContentIndex::new("earth"))
					.unwrap()
					.clone(),
			),
		}
	}

	pub fn set_camera_aspect(&mut self, v: f32) {
		self.camera.aspect = v
	}

	pub fn step(&mut self, ct: &Content, status: PlayerStatus) {
		if self.input.get_v_scroll() != 0.0 {
			self.camera.zoom = (self.camera.zoom + self.input.get_v_scroll())
				.clamp(ct.config.zoom_min, ct.config.zoom_max);
		}

		if status.pos.is_some() {
			self.camera.pos = status.pos.unwrap();
		}
	}
}
