use galactica_content::{resolve_edge_as_edge, Content};
use log::error;
use rhai::{FnNamespace, FuncRegistration, ImmutableString, Module};
use std::{cell::RefCell, rc::Rc, sync::Arc};

use crate::ui::{elements::UiSprite, UiElement, UiState};

use super::super::{Color, Rect};

pub fn build_sprite_module(ct_src: Arc<Content>, state_src: Rc<RefCell<UiState>>) -> Module {
	let mut module = Module::new();
	module.set_id("GalacticaSpriteModule");

	let state = state_src.clone();
	let ct = ct_src.clone();
	let _ = FuncRegistration::new("add")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(
			&mut module,
			move |name: ImmutableString, sprite: ImmutableString, rect: Rect| {
				let mut ui_state = state.borrow_mut();

				let sprite_handle = ct.sprites.get(&sprite.clone().into());
				if sprite_handle.is_none() {
					error!("made a sprite using an invalid source `{sprite}`");
					return;
				}

				if ui_state.elements.contains_key(&name) {
					error!("tried to make a sprite using an existing name `{name}`");
					return;
				}

				ui_state.names.push(name.clone());
				ui_state.elements.insert(
					name.clone(),
					UiElement::Sprite(UiSprite::new(
						name.clone(),
						sprite_handle.unwrap().clone(),
						rect,
					)),
				);
			},
		);

	let state = state_src.clone();
	let _ = FuncRegistration::new("exists")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString| {
			let mut ui_state = state.borrow_mut();
			match ui_state.get_mut_by_name(&name) {
				Some(UiElement::Sprite(_)) => true,
				_ => false,
			}
		});

	let state = state_src.clone();
	let _ = FuncRegistration::new("remove")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString| {
			let mut ui_state = state.borrow_mut();
			if ui_state.elements.contains_key(&name) {
				ui_state.elements.remove(&name).unwrap();
				ui_state.names.retain(|x| *x != name);
			} else {
				error!("called `sprite::remove` on an invalid name `{name}`")
			}
		});

	let state = state_src.clone();
	let ct = ct_src.clone();
	let _ = FuncRegistration::new("set_mask")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(
			&mut module,
			move |name: ImmutableString, mask: ImmutableString| {
				let mut ui_state = state.borrow_mut();

				match ui_state.get_mut_by_name(&name) {
					Some(UiElement::Sprite(x)) => {
						let m = ct.sprites.get(&mask.clone().into()).clone();
						if m.is_none() {
							error!("called `set_sprite_mask` with an invalid mask `{mask}`");
							return;
						}
						x.set_mask(m.cloned())
					}

					_ => {
						error!("called `sprite::set_mask` on an invalid name `{name}`")
					}
				}
			},
		);

	let state = state_src.clone();
	let _ = FuncRegistration::new("take_edge")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(
			&mut module,
			move |name: ImmutableString, edge_name: ImmutableString, duration: f32| {
				let mut ui_state = state.borrow_mut();

				match ui_state.get_mut_by_name(&name) {
					Some(UiElement::Sprite(x)) => {
						let sprite = x.anim.get_sprite();

						let edge =
							resolve_edge_as_edge(&sprite.sections, edge_name.as_str(), duration);
						let edge = match edge {
							Err(_) => {
								error!(
								"called `sprite::take_edge` on an invalid edge `{}` on sprite `{}`",
								edge_name, sprite.index
							);
								return;
							}
							Ok(s) => s,
						};

						x.anim.jump_to(&edge);
					}
					_ => {
						error!("called `sprite::take_edge` on an invalid name `{name}`")
					}
				}
			},
		);

	let state = state_src.clone();
	let _ = FuncRegistration::new("set_angle")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString, x: f32| {
			let mut ui_state = state.borrow_mut();
			match ui_state.get_mut_by_name(&name) {
				Some(UiElement::Sprite(s)) => s.set_angle(x),
				_ => {
					error!("called `sprite::set_angle` on an invalid name `{name}`")
				}
			}
		});

	let state = state_src.clone();
	let _ = FuncRegistration::new("set_rect")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString, x: Rect| {
			let mut ui_state = state.borrow_mut();
			match ui_state.get_mut_by_name(&name) {
				Some(UiElement::Sprite(s)) => s.set_rect(x),
				_ => {
					error!("called `sprite::set_rect` on an invalid name `{name}`")
				}
			}
		});

	let state = state_src.clone();
	let _ = FuncRegistration::new("set_color")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString, x: Color| {
			let mut ui_state = state.borrow_mut();
			match ui_state.get_mut_by_name(&name) {
				Some(UiElement::Sprite(s)) => s.set_color(x),
				_ => {
					error!("called `sprite::set_color` on an invalid name `{name}`")
				}
			}
		});

	let state = state_src.clone();
	let _ = FuncRegistration::new("preserve_aspect")
		.with_namespace(FnNamespace::Internal)
		.set_into_module(&mut module, move |name: ImmutableString, x: bool| {
			let mut ui_state = state.borrow_mut();
			match ui_state.get_mut_by_name(&name) {
				Some(UiElement::Sprite(s)) => s.set_preserve_aspect(x),
				_ => {
					error!("called `sprite::set_preserve_aspect` on an invalid name `{name}`")
				}
			}
		});
	return module;
}
